<?php

final class FlexPay {

    const SIGNATURE_KEY = 'SET_THIS_TO_YOUR_SIGNATURE_KEY'; // Set this to your signature key
    // The signature key can be found in the Verotel control center (

    const PURCHASE_URL = 'https://secure.verotel.com/order/purchase';
    const STATUS_URL   = 'https://secure.verotel.com/status/purchase';
    const VERSION      = '1';

    /**
     * FUNCTION get_signature:
     * -------------------------------
     * Returns SHA1 encoded signature
     * - IN $signatureKey - STRING
     * - IN $params - ASOC_ARRAY
     * - RETURNS STRING - generated SHA1 signature
     */
    public static function get_signature($signatureKey, $params) {
        $filtered = self::_filter_params($params);
        return self::_signature($signatureKey, $filtered);
    }


    /**
     * FUNCTION validate_signature:
     * ------------------------------
     * Validates signature
     * - IN $signatureKey - STRING
     * - IN $params - ASOC_ARRAY (just allowed params with signature)
     * - RETURNS BOLEAN - true/false
     */
    public static function validate_signature($signatureKey, $params) {
        $sign1 = strtolower($params['signature']);
        unset($params['signature']);
        $sign2 = self::get_signature($signatureKey, $params);
        return ($sign1 === $sign2) ? true : false;
    }


    /**
     * FUNCTION get_purchase_URL:
     * -----------------------------
     * - IN $signatureKey - STRING
     * - IN $params - ASOC_ARRAY
     * - RETURNS STRING - purchase_URL
     */
    public static function get_purchase_URL($signatureKey, $params) {
        if (!isset($signatureKey) || !is_string($signatureKey) || empty($signatureKey)) {
            throw new Exception("no signatureKey given");
        }

        if (!isset($params) || empty($params)) {
            throw new Exception("no params given");
        }

        if (!is_array($params)) {
            throw new Exception("invalid params");
        }

        return self::_generate_URL(self::PURCHASE_URL, $signatureKey, $params);
    }


    /**
     * FUNCTION get_status_URL:
     * ----------------------------
     * - IN $signatureKey - STRING
     * - IN $params - ASOC_ARRAY
     * - RETURNS STRING - status_URL
     */
    public static function get_status_URL($signatureKey, $params) {
        if (!isset($signatureKey) || !is_string($signatureKey) || empty($signatureKey)) {
            throw new Exception("no signatureKey given");
        }

        if (!isset($params) || empty($params)) {
            throw new Exception("no params given");
        }

        if (!is_array($params)) {
            throw new Exception("invalid params");
        }

        return self::_generate_URL(self::STATUS_URL, $signatureKey, $params);
    }


    ######## PRIVATE ############################

    /**
     * Common furnction for generating signature
     * - IN $signatureKey - STRING
     * - IN $params - ASOC_ARRAY
     */
    private static function _signature($signatureKey, $params) {
        $outArray = array($signatureKey);
        ksort($params, SORT_REGULAR);
        foreach ($params as $key => $value) {
            array_push($outArray, "$key=$value");
        }

        return strtolower(sha1(join(":", $outArray)));
    }


    /**
     * Returns url
     * - IN $baseURL - STRING (url : http://www.xyz.com)
     * - IN $signatureKey  - STRING
     * - IN $params  - ASOC_ARRAY (URL params)
     */
    private static function _generate_URL($baseURL, $signatureKey, $params) {
        $params['version'] = self::VERSION;

        ksort($params, SORT_REGULAR);
        $outArray = array();
        foreach ($params as $key => $value) {
            if($value !== "") {
                $outArray[$key] = $value;
            }
        }

        $signature = self::get_signature($signatureKey, $outArray);
        $outArray['signature'] = $signature;

        return self::_build_URL($baseURL, $outArray);
    }


    /**
     * Returns URL string
     * - IN $baseURL - STRING (url : http://www.xyz.com)
     * - IN $params  - ASOC_ARRAY - (URL params)
     */
    private static function _build_URL($baseURL, $params) {
        $arr = array();

        foreach ($params as $key => $value) {
            $arr[] = "$key=" . urlencode($value);
        }
        return $baseURL . "?" . join("&", $arr);
    }


    /**
     * Returns filtered parameters assoc-array
     * - IN $params - ASOC_ARRAY (unfiltered URL params)
     */
    private static function _filter_params($params) {

        $keys = array_keys($params);
        $filtered = array();
        $regexp = '/^(
            version
            | shopID
            | price(Amount|Currency)
            | description
            | referenceID
            | saleID
            | custom\w*
            )$/x';

        foreach ($keys as $key) {
            if (preg_match($regexp, $key)) {
                $filtered[$key] = $params[$key];
            }
        }

        return $filtered;
    }
}

?>
